package com.zanxiang.game.back.serve.rpc.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.game.back.base.pojo.dto.*;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.base.rpc.ITtAppBackRpc;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameOceanengineAppOrderLogService;
import com.zanxiang.game.back.serve.service.IGameOceanengineAppRoleRegisterLogService;
import com.zanxiang.game.back.serve.service.IGameOceanengineAppUserLogService;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Objects;

@Slf4j
@DubboService
public class TtAppBackRpcImpl implements ITtAppBackRpc {

    @Autowired
    private IGameBackPolicyService gameBackPolicyService;
    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;
    @Autowired
    private IGameOceanengineAppUserLogService gameOceanengineAppUserLogService;
    @Autowired
    private IGameOceanengineAppRoleRegisterLogService gameOceanengineAppRoleRegisterLogService;
    @Autowired
    private IGameOceanengineAppOrderLogService gameOceanengineAppOrderLogService;

    @Override
    public ResultVO<Boolean> userActiveReport(TtUserActiveAppRpcDTO dto) {
        log.error("头条 APP用户回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getMac())
                && StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }

        GameOceanengineAppUserLog userLog = GameOceanengineAppUserLog.builder()
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .gameId(dto.getGameId())
                .accountId(dto.getAccountReport().getAccountId())
                .agentKey(dto.getAgentKey())
                .activeTime(dto.getActiveTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .build();
        gameOceanengineAppUserLogService.save(userLog);
        return ResultVO.ok(gameOceanengineAppUserLogService.callback(userLog, Boolean.FALSE));
    }

    @Override
    public ResultVO<Boolean> orderReport(TtOrderAppRpcDTO dto) {
        if (Objects.equals(OrderStatusEnum.SUCCESS_PAY.getValue(), dto.getOrderStatus())) {
            log.error("头条 APP订单回传收到：{}。", JsonUtil.toString(dto));
        } else {
            log.error("头条 APP订单回传收到：{}。订单未支付，直接过滤", JsonUtil.toString(dto));
            return ResultVO.ok(true);
        }
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getMac())
                && StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameOceanengineAppOrderLog orderLog = GameOceanengineAppOrderLog.builder()
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .gameId(dto.getGameId())
                .backPolicyId(dto.getBackPolicyId())
                .accountId(dto.getAccountReport().getAccountId())
                .agentKey(dto.getAgentKey())
                .regTime(dto.getRegTime())
                .orderNo(dto.getOrderId())
                .amount(dto.getRechargeMoney())
                .orderStatus(dto.getOrderStatus())
                .createTime(dto.getCreateTime())
                .payTime(dto.getPayTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .build();
        boolean isFirstOrder = gameOceanengineAppOrderLogService.getOne(new LambdaQueryWrapper<GameOceanengineAppOrderLog>()
                .eq(GameOceanengineAppOrderLog::getGameId, orderLog.getGameId())
                .eq(GameOceanengineAppOrderLog::getMac, orderLog.getMac())
                .eq(GameOceanengineAppOrderLog::getImei, orderLog.getImei())
                .eq(GameOceanengineAppOrderLog::getOaid, orderLog.getOaid())
                .eq(GameOceanengineAppOrderLog::getAndroidId, orderLog.getAndroidId())
                .eq(GameOceanengineAppOrderLog::getIdfa, orderLog.getIdfa())
                .eq(GameOceanengineAppOrderLog::getCaid, orderLog.getCaid())
                .eq(GameOceanengineAppOrderLog::getAccountId, orderLog.getAccountId())
                .eq(GameOceanengineAppOrderLog::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                .last("limit 1")
        ) == null;
        orderLog.setIsFirstOrder(isFirstOrder);
        gameOceanengineAppOrderLogService.save(orderLog);
        return ResultVO.ok(gameOceanengineAppOrderLogService.callback(orderLog));
    }

    @Override
    public ResultVO<Boolean> roleRegisterReport(TtRoleRegisterAppRpcDTO dto) {
        log.error("头条 APP创角回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getMac())
                && StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameOceanengineAppRoleRegisterLog roleRegisterLog = GameOceanengineAppRoleRegisterLog.builder()
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .gameId(dto.getGameId())
                .accountId(dto.getAccountReport().getAccountId())
                .agentKey(dto.getAgentKey())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .registerTime(dto.getRegisterTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .build();
        gameOceanengineAppRoleRegisterLogService.save(roleRegisterLog);
        return ResultVO.ok(gameOceanengineAppRoleRegisterLogService.callback(roleRegisterLog));
    }

    @Override
    public ResultVO<Boolean> userBackQuery(TtAppUserBackQueryRpcDTO dto) {
        return ResultVO.ok(Boolean.TRUE);
    }

    @Override
    public ResultVO<Boolean> orderBackQuery(TtAppOrderBackQueryRpcDTO dto) {
        GameOceanengineAppOrderLog orderLog = gameOceanengineAppOrderLogService.getOne(new LambdaQueryWrapper<GameOceanengineAppOrderLog>()
                .eq(GameOceanengineAppOrderLog::getOrderNo, dto.getOrderId())
                .eq(GameOceanengineAppOrderLog::getGameId, dto.getGameId())
                .eq(GameOceanengineAppOrderLog::getAgentKey, dto.getAgentKey())
                .eq(GameOceanengineAppOrderLog::getAccountId, dto.getAccountReport().getAccountId())
                .eq(GameOceanengineAppOrderLog::getMac, dto.getMac())
                .eq(GameOceanengineAppOrderLog::getImei, dto.getImei())
                .eq(GameOceanengineAppOrderLog::getOaid, dto.getOaid())
                .eq(GameOceanengineAppOrderLog::getAndroidId, dto.getAndroidId())
                .eq(GameOceanengineAppOrderLog::getIdfa, dto.getIdfa())
                .eq(GameOceanengineAppOrderLog::getCaid, dto.getCaid())
        );
        if  (orderLog == null) {
            return ResultVO.ok(Boolean.FALSE);
        }
        return ResultVO.ok(BackStatusEnum.getByValue(orderLog.getBackStatus()) ==  BackStatusEnum.SUCCESS);
    }

    @Override
    public ResultVO<Boolean> roleRegisterBackQuery(TtAppRoleRegisterBackQueryRpcDTO dto) {
        return ResultVO.ok(Boolean.TRUE);
    }
}
