package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameOceanengineAppUserLogMapper;
import com.zanxiang.game.back.serve.oceanengine.MiniGameCallback;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineAppBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineAppUserLog;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameOceanengineAppBackLogService;
import com.zanxiang.game.back.serve.service.IGameOceanengineAppUserLogService;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

@Slf4j
@Service
public class GameOceanengineAppUserLogServiceImpl extends ServiceImpl<GameOceanengineAppUserLogMapper, GameOceanengineAppUserLog>
implements IGameOceanengineAppUserLogService {

    @Autowired
    private IGameOceanengineAppBackLogService gameOceanengineAppBackLogService;
    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean callback(GameOceanengineAppUserLog userLog, boolean mustBack) {
        BackStatusEnum backStatus = doCallback(userLog, mustBack);
        return update(new LambdaUpdateWrapper<GameOceanengineAppUserLog>()
                .set(GameOceanengineAppUserLog::getBackStatus, backStatus.getBackStatus())
                .eq(GameOceanengineAppUserLog::getId, userLog.getId())
        );
    }

    private BackStatusEnum doCallback(GameOceanengineAppUserLog userLog, boolean mustBack) {
        // 是否已经回传过
        if (!mustBack && gameOceanengineAppBackLogService.count(new LambdaQueryWrapper<GameOceanengineAppBackLog>()
                .eq(GameOceanengineAppBackLog::getGameId, userLog.getGameId())
                .eq(GameOceanengineAppBackLog::getMac, userLog.getMac())
                .eq(GameOceanengineAppBackLog::getImei, userLog.getImei())
                .eq(GameOceanengineAppBackLog::getOaid, userLog.getOaid())
                .eq(GameOceanengineAppBackLog::getAndroidId, userLog.getAndroidId())
                .eq(GameOceanengineAppBackLog::getIdfa, userLog.getIdfa())
                .eq(GameOceanengineAppBackLog::getCaid, userLog.getCaid())
                .eq(GameOceanengineAppBackLog::getAccountId, userLog.getAccountId())
                .eq(GameOceanengineAppBackLog::getEventType, MiniGameCallback.EventType.ACTIVE.getEventType())
                .eq(GameOceanengineAppBackLog::getBackStatus, BackStatusEnum.SUCCESS.getBackStatus())
        ) > 0) {
            return BackStatusEnum.NO;
        }
        BackStatusEnum backStatus = BackStatusEnum.SUCCESS;
        gameOceanengineAppBackLogService.save(GameOceanengineAppBackLog.builder()
                .mac(userLog.getMac())
                .imei(userLog.getImei())
                .oaid(userLog.getOaid())
                .androidId(userLog.getAndroidId())
                .idfa(userLog.getIdfa())
                .caid(userLog.getCaid())
                .gameId(userLog.getGameId())
                .accountId(userLog.getAccountId())
                .eventType(String.valueOf(MiniGameCallback.EventType.ACTIVE.getEventType()))
                .eventLogId(userLog.getId())
                .createTime(LocalDateTime.now())
                .backStatus(backStatus.getBackStatus())
                .errMsg(null)
                .build());
        return backStatus;
    }
}
